@@ -109,7 +109,7 @@
               (eq ($sign y) '$neg)
               (zerop1 (sub (simplify (list '(%truncate) y)) y)))
          ;; Negative integer or a real representation of a negative integer.
-         (merror "factorial(~:M) is undefined." y))
+         (merror (intl:gettext "factorial: factorial of negative integer ~:M not defined.") y))
         ((or (floatp y)             
              ($bfloatp y)
              (and (not (integerp y))
@@ -358,7 +358,7 @@
                     (or (<= c a) (= b 0))
                     (<= b (/ a c)))
              (gfact a b c)
-             (merror "Bad argument to `genfact'")))
+             (merror (intl:gettext "genfact: generalized factorial not defined for given arguments."))))
 	  (t (eqtest (list '(%genfact) a
 			   (if (and (not (atom b))
 				    (eq (caar b) '$floor))
@@ -427,7 +427,7 @@ summation when necessary."
 	(ind (cadr l))
 	(lis (meval (caddr l)))
 	(ans 0))
-    (or (symbolp ind) (merror "Second argument not a variable ~M" ind))
+    (or (symbolp ind) (merror (intl:gettext "lsum: second argument must be a variable; found ~M") ind))
     (cond (($listp lis)
 	   (loop for v in (cdr lis)
 	      with lind = (cons ind nil)
@@ -448,7 +448,7 @@ summation when necessary."
 ; some simplifications not handled by the revised code. -- Robert Dodier
 
 (defun simpsum1-save (exp i lo hi)
-  (cond ((not (symbolp i)) (merror "Improper index to `sum':~%~M" i))
+  (cond ((not (symbolp i)) (merror (intl:gettext "sum: index must be a symbol; found ~M") i))
 	((equal lo hi) (mbinding ((list i) (list hi)) (meval exp)))
 	((and (atom exp)
 	      (not (eq exp i))
@@ -486,7 +486,7 @@ summation when necessary."
 (defun dosum (expr ind low hi sump &key (evaluate-summand t))
   (setq low (ratdisrep low) hi (ratdisrep hi)) ;; UGH, GAG WITH ME A SPOON
   (if (not (symbolp ind))
-      (merror "~:M: bad index ~M (must be a symbol)~%" (if sump '$sum '$product) ind))
+      (merror (intl:gettext "~:M: index must be a symbol; found ~M") (if sump '$sum '$product) ind))
   (unwind-protect
        (prog (u *i lind l*i *hl)
 	  (setq lind (cons ind nil))
@@ -557,7 +557,7 @@ summation when necessary."
   (let ((ind (cadr l)))
     (if (mquotep ind) (setq ind (cadr ind)))
     (if (not (symbolp ind))
-      (merror "~:M: bad index ~M (must be a symbol)~%" op ind))
+      (merror (intl:gettext "~:M: index must be a symbol; found ~M") op ind))
     (let ((low (caddr l))
 	  (hi (cadddr l)))
       (list (mevalsumarg (car l) ind low hi)
@@ -695,14 +695,14 @@ summation when necessary."
 
 (defun simpprod1-save (exp i lo hi)
   (let (u)
-    (cond ((not (symbolp i)) (merror "Bad index to product:~%~M" i))
+    (cond ((not (symbolp i)) (merror (intl:gettext "product: index must be a symbol; found ~M") i))
 	  ((alike1 lo hi)
 	   (let ((valist (list i)))
 	     (mbinding (valist (list hi))
 		       (meval exp))))
 	  ((eq ($sign (setq u (m- hi lo))) '$neg)
 	   (cond ((eq ($sign (add2 u 1)) '$zero) 1)
-		 (t (merror "Lower bound to product is > upper bound."))))
+		 (t (merror (intl:gettext "product: lower bound ~M greater than upper bound ~M") lo hi))))
 	  ((atom exp)
 	   (cond ((null (eq exp i))
 		  (power* exp (list '(mplus) hi 1 (list '(mtimes) -1 lo))))
@@ -877,7 +877,7 @@ summation when necessary."
 (defun linearize3 (w e x)
   (let (w1)
     (if (and (member w '($inf $minf $infinity) :test #'eq) (safe-zerop x))
-	(merror "Undefined form 0*inf:~%~M" e))
+	(merror (intl:gettext "LINEARIZE3: undefined form 0*inf: ~M") e))
     (setq w (mul2 (simplifya x t) w))
     (cond ((or (atom w) (getl (caar w) '($outative $linear))) (setq w1 1))
 	  ((eq (caar w) 'mtimes)
@@ -967,13 +967,13 @@ summation when necessary."
 
 (defmfun evenfun (e z)
   (if (or (null (cdr e)) (cddr e))
-      (merror "Even function called with wrong number of arguments:~%~M" e))
+      (merror (intl:gettext "Function declared 'even' takes exactly one argument; found ~M") e))
   (let ((arg (simpcheck (cadr e) z)))
     (oper-apply (list (car e) (if (mminusp arg) (neg arg) arg)) t)))
 
 (defmfun oddfun (e z)
   (if (or (null (cdr e)) (cddr e))
-      (merror "Odd function called with wrong number of arguments:~%~M" e))
+      (merror (intl:gettext "Function declared 'odd' takes exactly one argument; found ~M") e))
   (let ((arg (simpcheck (cadr e) z)))
     (if (mminusp arg) (neg (oper-apply (list (car e) (neg arg)) t))
 	(oper-apply (list (car e) arg) t))))
