@@ -835,11 +835,17 @@
 	   (setq n (nconc n (copy-tree a)))))))
 
 (defmfun $qunit (n)
-  (let ((l (sqcont n)))
-    (list '(mplus) (pelso1 l 0 1)
-	  (list '(mtimes)
-		(list '(mexpt) n '((rat) 1 2))
-		(pelso1 l 1 0)))))
+  (let ((isqrtn ($isqrt n)))
+    (when (or (not (integerp n))
+              (minusp n)
+              (= (* isqrtn isqrtn) n))
+      (merror
+        (intl:gettext "qunit: Argument must be a positive non quadratic integer.")))
+    (let ((l (sqcont n)))
+      (list '(mplus) (pelso1 l 0 1)
+            (list '(mtimes)
+                  (list '(mexpt) n '((rat) 1 2))
+                  (pelso1 l 1 0))))))
 
 (defun pelso1 (l a b)
   (do ((i l (cdr i))) (nil)
@@ -1107,9 +1113,9 @@
 		   (integerp c))
 	      (progn
 		(adsum (m* y (m^ 2 (car n))))
-		(when (member (asksign (m- h1 (car n))) '($zero $negative) :test #'eq)
+		(when (member (asksign (m- (m+ h1 c) (car n))) '($zero $negative) :test #'eq)
 		  (adsum (m* -1 y (dosum (list '(%binomial) (car n) *var*)
-					 *var* (m+ h1 1) (car n) t :evaluate-summand nil))))
+					 *var* (m+ h1 c 1) (car n) t :evaluate-summand nil))))
 		(when (> c 0)
 		  (adsum (m* -1 y (dosum (list '(%binomial) (car n) *var*)
 					 *var* 0 (m- c 1) t :evaluate-summand nil)))))
