@@ -117,7 +117,7 @@
 
 ;;Remainder of page is update from F302 --gsb
 
-;;Used only in COMM2 (AT), and below.
+;;Used in COMM2 (AT), limit, and below.
 (defvar dummy-variable-operators '(%product %sum %laplace %integrate %limit %at))
 
 (defun subst1 (z)			; Y is an atom
@@ -126,7 +126,7 @@
 	((eq (caar z) 'bigfloat) z)
 	((and (eq (caar z) 'rat) (or (equal y (cadr z)) (equal y (caddr z))))
 	 (div (subst1 (cadr z)) (subst1 (caddr z))))
-	((at-substp z) z)
+	((at-substp z) (subst-except-second-arg x y z))
 	((and (eq y t) (eq (caar z) 'mcond))
 	 (list (cons (caar z) nil) (subst1 (cadr z)) (subst1 (caddr z))
 	       (cadddr z) (subst1 (car (cddddr z)))))
@@ -168,6 +168,20 @@
 		   (t (cons (cons (caar z) nil) (cons x (cdr tail)))))))
 	  (t (recur-apply #'subst2 z)))))
 
+;; replace y with x in z, but leave z's second arg unchanged.
+;; This is for cases like at(integrate(x, x, a, b), [x=3])
+;; where second arg of integrate binds a new variable x,
+;; and we do not wish to subst 3 for x inside integrand.
+(defun subst-except-second-arg (x y z)
+  (append 
+   (list (car z)
+	 (if (eq y (third z))	; if (third z) is new var that shadows y
+	     (second z)		; leave (second z) unchanged
+	   (subst1 (second z)))	; otherwise replace y with x in (second z)
+	 (third z))		; never change integration var
+   (mapcar (lambda (z) (subst1 z))	; do subst in limits of integral
+	   (cdddr z))))
+
 (declare-top (unspecial x y oprx opry negxpty timesp))
 
 (defmfun subst0 (new old)
@@ -364,10 +378,13 @@
     (if (depends u x) (return t))))
 
 (defmfun sdiff (e x) ; The args to SDIFF are assumed to be simplified.
+  ;; Remove a special representation from the variable of differentiation
+  (setq x (specrepcheck x))
   (cond ((alike1 e x) 1)
 	((mnump e) 0)
 	((or (atom e) (member 'array (cdar e) :test #'eq)) (chainrule e x))
 	((eq (caar e) 'mrat) (ratdx e x))
+        ((eq (caar e) 'mpois) ($poisdiff e x)) ; Poisson series
 	((eq (caar e) 'mplus) (addn (sdiffmap (cdr e) x) t))
 	((mbagp e) (cons (car e) (sdiffmap (cdr e) x)))
 	((member (caar e) '(%sum %product) :test #'eq) (diffsumprod e x))
@@ -398,8 +415,9 @@
 	((eq (caar e) '%integrate) (diffint e x))
 	((eq (caar e) '%laplace) (difflaplace e x))
 	((eq (caar e) '%at) (diff-%at e x))
-	((member (caar e) '(%realpart %imagpart) :test #'eq)
-	 (list (cons (caar e) nil) (sdiff (cadr e) x)))
+; This rule is not correct. We cut it out.
+;	((member (caar e) '(%realpart %imagpart) :test #'eq)
+;	 (list (cons (caar e) nil) (sdiff (cadr e) x)))
 	((and (eq (caar e) 'mqapply)
 	      (eq (caaadr e) '$%f))
 	 ;; Handle %f, hypergeometric function
@@ -420,31 +438,63 @@
 	(t (sdiffgrad e x))))
 
 (defun sdiffgrad (e x)
-  (let ((fun (caar e)) grad args)
+  (let ((fun (caar e)) grad args result)
     (cond ((and (eq fun 'mqapply) (oldget (caaadr e) 'grad))
-	   (sdiffgrad (cons (cons (caaadr e) nil) (append (cdadr e) (cddr e)))
-		      x))
+           ;; Change the array function f[n](x) to f(n,x), call sdiffgrad again.
+           (setq result
+	         (sdiffgrad (cons (cons (caaadr e) nil) 
+	                          (append (cdadr e) (cddr e)))
+		            x))
+           ;; If noun form for f(n,x), adjust the noun form for f[n](x)
+           (if (isinop result '%derivative)
+               (if (not (depends e x))
+                   0
+                   (diff%deriv (list e x 1)))
+               result))
 
 	  ;; extension for pdiff.
 	  ((and (get '$pderivop 'operators) (sdiffgrad-pdiff e x)))
 
+	  ;; two line extension for hypergeometric.
+	  ((and (equal fun '$hypergeometric) (get '$hypergeometric 'operators))
+	   (diff-hypergeometric (second e) (third e) (fourth e) x))
+
 	  ((or (eq fun 'mqapply) (null (setq grad (oldget fun 'grad))))
 	   (if (not (depends e x)) 0 (diff%deriv (list e x 1))))
 	  ((not (= (length (cdr e)) (length (car grad))))
-	   (merror (intl:gettext "~:M: expected exactly ~M arguments.") fun (length (car grad))))
-	  (t (setq args (sdiffmap (cdr e) x))
-	     (addn (mapcar
-		    #'mul2
-		    (cdr (substitutel
-			  (cdr e) (car grad)
-			  (do ((l1 (cdr grad) (cdr l1))
-			       (args args (cdr args)) (l2))
-			      ((null l1) (cons '(mlist) (nreverse l2)))
-			    (setq l2 (cons (cond ((equal (car args) 0) 0)
-						 (t (car l1)))
-					   l2)))))
-		    args)
-		   t)))))
+	   (merror (intl:gettext "~:M: expected exactly ~M arguments.") 
+	           fun 
+	           (length (car grad))))
+	  (t
+           (setq args (sdiffmap (cdr e) x))
+           (setq result
+                 (addn
+                   (mapcar 
+                     #'mul2
+                     (cdr 
+                       (substitutel
+                         (cdr e) 
+                         (car grad)
+                         (do ((l1 (cdr grad) (cdr l1))
+                              (args args (cdr args)) 
+                              (l2))
+                             ((null l1) (cons '(mlist) (nreverse l2)))
+                           (setq l2
+                                 (cons (cond ((equal (car args) 0) 0)
+                                             ((functionp (car l1))
+                                              ;; Evaluate a lambda expression
+                                              ;; given as a derivative.
+                                              (apply (car l1) (cdr e)))
+                                             (t (car l1)))
+                                       l2)))))
+                     args)
+                   t))
+           (if (or (null result) (not (freeof nil result)))
+               ;; A derivative has returned NIL. Return a noun form.
+               (if (not (depends e x))
+                   0 
+                   (diff%deriv (list e x 1)))
+               result)))))
 
 (defun sdiffmap (e x)
   (mapcar #'(lambda (term) (sdiff term x)) e))
@@ -516,19 +566,27 @@
 	  (mabs ((mtimes) x ((mexpt) ((mabs) x) -1)))
 	  (%erf ((mtimes) 2 ((mexpt) $%pi ((rat) -1 2))
 		 ((mexpt) $%e ((mtimes) -1 ((mexpt) x 2)))))
-	  ;;	   ($LI2 ((MTIMES) -1 ((%LOG) ((MPLUS) 1 ((MTIMES) -1 X))) ((MEXPT) X -1)))
-	  ($ei ((mtimes) ((mexpt) x -1) ((mexpt) $%e x))))))
+	  )))
 
 (defprop $atan2 ((x y) ((mtimes) y ((mexpt) ((mplus) ((mexpt) x 2) ((mexpt) y 2)) -1))
 		 ((mtimes) -1 x ((mexpt) ((mplus) ((mexpt) x 2) ((mexpt) y 2)) -1)))
   grad)
 
-(defprop $li ((n x) ((%derivative) ((mqapply) (($li array) n) x) n 1)
-	      ((mtimes) ((mqapply) (($li array) ((mplus) -1 n)) x) ((mexpt) x -1)))
+(defprop $li 
+  ((n x)
+; Do not put a noun form on the property list, but NIL.
+; SDIFFGRAD generates the noun form.
+;   ((%derivative) ((mqapply) (($li array) n) x) n 1)
+   nil
+   ((mtimes) ((mqapply) (($li array) ((mplus) -1 n)) x) ((mexpt) x -1)))
   grad)
 
-(defprop $psi ((n x) ((%derivative) ((mqapply) (($psi array) n) x) n 1)
-	       ((mqapply) (($psi array) ((mplus) 1 n)) x))
+(defprop $psi 
+  ((n x)
+; Do not put a noun form on the property list, but NIL.
+; SDIFFGRAD generates the noun form.
+   nil
+   ((mqapply) (($psi array) ((mplus) 1 n)) x))
   grad)
 
 (defmfun atvarschk (argl)
