@@ -139,7 +139,7 @@
     (if (or (mopp fun) (eq fun 'mqapply)) (improper-arg-err exp '$atvalue))
     (atvarschk vl)
     (do ((vl1 vl (cdr vl1)) (l atvars (cdr l))) ((null vl1))
-      (if (and (symbolp (car vl1)) (not (mget (car vl1) '$constant)))
+      (if (and (symbolp (car vl1)) (not (kindp (car vl1) '$constant)))
 	  (setq val (maxima-substitute (car l) (car vl1) val))
 	  (improper-arg-err (cons '(mlist) vl) '$atvalue)))
     (setq eqs (if (eq (caar eqs) 'mequal) (list eqs) (cdr eqs)))
@@ -240,9 +240,19 @@
 		   ((logconcoeffp (car x)) (setq decints (cons (car x) decints)))
 		   (t (setq notlogs (cons (car x) notlogs))))))))
 
-(defun lgcsimp (e)		;; this is to simplify
-  (let (($logexpand nil))	;; log(%e) -> 1 and log(%e^2) -> 2
-    (simpln `((%log) ,e) 1 t)))
+;(defun lgcsimp (e)		;; this is to simplify
+;  (let (($logexpand nil))	;; log(%e) -> 1 and log(%e^2) -> 2
+;    (simpln `((%log) ,e) 1 t)))
+
+(defun lgcsimp (e)
+  (cond ((atom e)
+         ;; e.g. log(1) -> 0, or log(%e) -> 1
+         (simplify (list '(%log) e)))
+        ((and (mexptp e) (eq (cadr e) '$%e))
+         ;; log(%e^expr) -> expr
+         (simplify (list '(%log) e)))
+        (t
+         (list '(%log simp) e))))
 
 (defun lgcsimplep (e)
   (and (eq (caar e) 'mplus)
@@ -300,14 +310,22 @@
 (defun rtcon (e)
   (cond ((atom e) e)
 	((eq (caar e) 'mtimes)
-	 (if (and (not (free e '$%i))
-		  (let ((num ($num e)))
-		    (and (not (alike1 e num))
-			 (or (eq num '$%i)
-			     (and (not (atom num)) (member '$%i num :test #'eq)
-				  (member '$%i (rtcon num) :test #'eq))))))
-	     (setq e (list* (car e) -1 '((mexpt) -1 ((rat simp) -1 2))
-			    (delete '$%i (copy-list (cdr e)) :count 1 :test #'eq))))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+; This code does the substitution %i -> (-1)*(-1)^(-1/2) for expressions
+; like %i/expr. In most cases this causes a wrong result, e.g. 
+; bug ID: 2820202 - rootscontract(%i/2) --> -%i/2. The only case this codes 
+; works is for expressions like %i/sqrt(x) --> -1/sqrt(x). It is doubtful that 
+; this simplification is useful.
+; To get correct results in general, this code is commented out. DK 08/2009.
+; 
+;	 (if (and (not (free e '$%i))
+;		  (let ((num ($num e)))
+;		    (and (not (alike1 e num))
+;			 (or (eq num '$%i)
+;			     (and (not (atom num)) (member '$%i num :test #'eq)
+;				  (member '$%i (rtcon num) :test #'eq))))))
+;	     (setq e (list* (car e) -1 '((mexpt) -1 ((rat simp) -1 2))
+;			    (delete '$%i (copy-list (cdr e)) :count 1 :test #'eq))))
 	 (do ((x (cdr e) (cdr x)) (roots) (notroots) (y))
 	     ((null x)
 	      (cond ((null roots) (subst0 (cons '(mtimes) (nreverse notroots)) e))
@@ -408,6 +426,19 @@
     (setq y (simpcheck (cadr e) z) x (simpcheck (caddr e) z))
     (cond ((and (zerop1 y) (zerop1 x))
 	   (merror (intl:gettext "atan2: atan2(0,0) is undefined.")))
+          ;; Simplifify infinities.
+          ((or (eq x '$inf)
+               (alike1 x '((mtimes) -1 $minf)))
+           ;; The argument x is inf or -minf.
+           0)
+          ((or (eq x '$minf)
+               (alike1 x '((mtimes) -1 $inf)))
+           ;; The argument x is minf or -inf. Determine the sign of y.
+           ;; When unknown, return a noun form.
+           (setq signy ($sign y))
+           (cond ((eq signy '$pos) '$%pi)
+                 ((eq signy '$neg) (mul -1 '$%pi))
+                 (t (eqtest (list '($atan2) y x) e))))
 	  ( ;; float contagion
 	   (and (or (numberp x) (ratnump x)) ; both numbers
 		(or (numberp y) (ratnump y)) ; ...but not bigfloats
@@ -438,7 +469,8 @@
 		      ;; *Atan* should handle sqrt(3) etc., so all cases will work
 		      ((and (equal y 1) (alike1 x '((mexpt simp) 3 ((rat simp) 1 2))))
 		       (mul2* '((rat simp) 1 6) '$%pi)))))
-	  ($logarc (logarc '%atan (div y x)))
+          ($logarc
+           (logarc '%atan2 (list ($logarc y) ($logarc x))))
 	  ((and $trigsign (mminusp* y))
 	   (neg (simplifya (list '($atan2) (neg y) x) t)))
 					; atan2(y,x) = atan(y/x) + pi sign(y) (1-sign(x))/2
@@ -455,7 +487,11 @@
 	  (t (eqtest (list '($atan2) y x) e)))))
 
 (defun atan2negp (e) (eq ($sign e) '$neg))
-(defun atan2posp (e) (eq ($sign e) '$pos))
+
+(defun atan2posp (e)
+  ;; Include $pz for this check. With this extension expessions like
+  ;; abs(z) will be positive and further simplified.
+  (member ($sign e) '($pos $pz)))
 
 ;;;; ARITHF
 
