@@ -20,12 +20,12 @@
 (load-macsyma-macros rzmac)
 
 (declare-top (special $nointegrate $lhospitallim $tlimswitch $limsubst
-		      $abconvtest complex-limit plogabs $intanalysis))
+		      $abconvtest complex-limit plogabs))
 
 
 (setq $demoivre nil rsn* nil $nointegrate nil $lhospitallim 4
       $tlimswitch t $limsubst nil $abconvtest nil
-      complex-limit nil plogabs nil $intanalysis t)
+      complex-limit nil plogabs nil)
 
 (defmvar %p%i '((mtimes) $%i $%pi))
 (defmvar fourth%pi '((mtimes) ((rat simp) 1 4) $%pi))
@@ -339,22 +339,22 @@
 	     ((equal j '(-1 2)) -1)
 	     ((or (equal j '(1 3))
 		  (equal j '(2 3)))
-	      sqrt3//2)
+              (div ($sqrt 3) 2))
 	     ((or (equal j '(-1 3))
 		  (equal j '(-2 3)))
-	      -sqrt3//2)
+             (div ($sqrt 3) -2))
 	     ((or (equal j '(1 6))
 		  (equal j '(5 6)))
-	      '((rat) 1 2))
+	      '((rat simp) 1 2))
 	     ((or (equal j '(-1 6))
 		  (equal j '(-5 6)))
-	      '((rat) -1 2))
+	      '((rat simp) -1 2))
 	     ((or (equal j '(1 4))
 		  (equal j '(3 4)))
-	      sqrt2//2)
+              (div 1 ($sqrt 2)))
 	     ((or (equal j '(-1 4))
 		  (equal j '(-3 4)))
-	      -sqrt2//2)
+             (div -1 ($sqrt 2)))
 	     (t (cond ((mnegp ang)
 		       (setq ang (timesk -1 ang) ep t)))
 		(setq ang (list '(mtimes simp)
