@@ -346,7 +346,7 @@ One extra decimal digit in actual representation for rounding purposes.")
     ;; 1.  The largest double-float is .999999 * 2^1024.  So if the
     ;; exponent is 1025 or higher, we have an overflow.
     (let ((e (+ exponent (- precision) *m machine-mantissa-precision)))
-      (if (>= (abs e) 1025)
+      (if (>= e 1025)
 	  (merror (intl:gettext "float: floating point overflow converting ~:M") l)
 	  (scale-float mantissa e)))))
 
@@ -733,10 +733,17 @@ One extra decimal digit in actual representation for rounding purposes.")
 ;;
 ;; %e = sum( 1/i! ,i,0,inf )
 ;;
-;; but sums up 1001 terms to one.
+;; but adds up k summands to one, for e.g. k=4 that means
+;;
+;;    1          1          1       1      1 + n*(1 + (n - 1)*(1 + (n - 2)))
+;; -------- + -------- + -------- + --  =  ---------------------------------
+;; (n - 3)!   (n - 2)!   (n - 1)!   n!                    n!
+;;
+;; The number of added summands should depend on the current precision. 
+;; k = isqrt(prec) seems to fit here.
 ;;
 (defun compe (prec)
-  (let (s h (n 1) d (k 1001))
+  (let (s h (n 1) d (k (isqrt prec))) 
      (setq h (ash 1 prec))
      (setq s h)
      (do ((i k (+ i k)))
@@ -1002,12 +1009,9 @@ One extra decimal digit in actual representation for rounding purposes.")
 	((not ($bfloatp p)) (list '(mexpt) p n))
 	((equal (cadr p) 0) ($bfloat 0))
 	((and (< (cadr p) 0) (ratnump n))
-	 ($bfloat
-	  ($expand (list '(mtimes)
-			 ($bfloat (let (($domain '$complex)
-					($m1pbranch t))
-				    (power -1 n)))
-			 (exptbigfloat (bcons (fpminus (cdr p))) n)))))
+	 (mul2 (let ($numer $float $keepfloat $ratprint)
+		 (power -1 n))
+	       (exptbigfloat (bcons (fpminus (cdr p))) n)))
 	((and (< (cadr p) 0) (not (integerp n)))
 	 (cond ((or (equal n 0.5) (equal n bfhalf))
 		(exptbigfloat p '((rat simp) 1 2)))
