@@ -68,7 +68,7 @@ When one changes, the other does too."
      (set (get var 'lisp-shadow) value)
      value)
     (t
-      (merror "Attempt to assign a non-string to ~:M" var))))
+      (merror (intl:gettext "assignment: must assign a string to ~:M; found: ~M") var value))))
 
 (defun print-directories ()
   (format t "maxima-prefix=~a~%" *maxima-prefix*)
@@ -274,7 +274,7 @@ When one changes, the other does too."
 		  (enough-namestring dir share-root)))
 	    file-list)))
 
-#+(or clisp ecl)
+#+clisp
 (defun share-subdirs-list ()
   ;; This doesn't work yet on windows.  Give up in that case and use
   ;; the default list.
@@ -301,6 +301,36 @@ When one changes, the other does too."
 	;; Sort in alphabetical order
 	(sort dir-list #'string-lessp))))
 
+#+ecl
+(defun share-subdirs-list ()
+  ;; This doesn't work yet on windows.  Give up in that case and use
+  ;; the default list.
+  (if (string= *autoconf-win32* "true")
+      (default-share-subdirs-list)
+      ;; The call to DIRECTORY is to get ecl to follow any symlinks so
+      ;; that the subsequent call to directory all start with the same
+      ;; initial path.
+      (let* ((share-root (first (directory (pathname (concatenate 'string *maxima-sharedir* "/")))))
+	     (dir-list (directory (merge-pathnames (make-pathname :directory '(:relative :wild-inferiors))
+						   share-root))))
+	;; dir-list contains all of the directories.  Remove stuff we
+	;; don't want like CVS directories.  Anything else?
+	(setf dir-list (delete-if #'(lambda (x)
+				      ;; Remove CVS directories
+				      (or (equal x share-root)
+					  (equal "CVS" (car (last (pathname-directory x))))))
+				  dir-list))
+	;; Now just want the part after the *maxima-sharedir*, and we want
+	;; strings.
+	(setf dir-list
+	      (mapcar #'(lambda (x)
+			  (let ((dir (make-pathname :directory (butlast (pathname-directory x))
+						    :name (car (last (pathname-directory x))))))
+			    (enough-namestring dir share-root)))
+		      dir-list))
+	;; Sort in alphabetical order
+	(sort dir-list #'string-lessp))))
+
 
 (defun default-share-subdirs-list ()
   ;; Default implementation.  Eventually this should go away.
@@ -365,6 +395,7 @@ When one changes, the other does too."
     "misc"
     "numeric"
     "orthopoly"
+    "hypergeometric"
     "physics"
     "simplification"
     "sym"
@@ -492,9 +523,10 @@ When one changes, the other does too."
 	    (combine-path *maxima-infodir* subdir-bit "maxima-index.lisp")))))
 
 (defun get-dirs (path)
-  #+(or :clisp :sbcl :ecl)
-  (directory (concatenate 'string (namestring path) "/*/"))
-  #-(or :clisp :sbcl :ecl)
+  #+(or :clisp :sbcl :ecl :openmcl)
+  (directory (concatenate 'string (namestring path) "/*/")
+	     #+openmcl :directories #+openmcl t)
+  #-(or :clisp :sbcl :ecl :openmcl)
   (directory (concatenate 'string (namestring path) "/*")))
 
 (defun unix-like-basename (path)
@@ -541,6 +573,10 @@ When one changes, the other does too."
   ;;    (mapc #'(lambda (x) (format t "\"~a\"~%" x)) (get-application-args))
   ;;    (terpri)
   (let ((maxima-options nil))
+    ;; Note: The current option parsing code expects every short
+    ;; option to have an equivalent long option.  No check is made for
+    ;; this, so please make sure this holds.  Or change the code in
+    ;; process-args in command-line.lisp.
     (setf maxima-options
 	  (list
 	   (make-cl-option :names '("-b" "--batch")
@@ -592,6 +628,10 @@ When one changes, the other does too."
 				       (list-cl-options maxima-options)
 				       (bye))
 			   :help-string "Display this usage message.")
+	   (make-cl-option :names '("--userdir")
+			   :argument "<directory>"
+			   :action nil
+			   :help-string "Use  <directory> for user directory (default is $HOME/.maxima)")
 	   (make-cl-option :names '("-l" "--lisp")
 			   :argument "<lisp>"
 			   :action nil
@@ -605,9 +645,9 @@ When one changes, the other does too."
 			   :action #'(lambda (file)
 				       #-sbcl (load file) #+sbcl (with-compilation-unit nil (load file)))
 			   :help-string "Preload <lisp-file>.")
-       (make-cl-option :names '("-q" "--quiet")
-	       :action #'(lambda () (declare (special *maxima-quiet*)) (setq *maxima-quiet* t))
-	       :help-string "Suppress Maxima start-up message.")
+	   (make-cl-option :names '("-q" "--quiet")
+			   :action #'(lambda () (declare (special *maxima-quiet*)) (setq *maxima-quiet* t))
+			   :help-string "Suppress Maxima start-up message.")
 	   (make-cl-option :names '("-r" "--run-string")
 			   :argument "<string>"
 			   :action #'(lambda (string)
@@ -637,10 +677,16 @@ When one changes, the other does too."
 				       ($quit))
 			   :help-string
 			   "Display the default installed version.")
-       (make-cl-option :names '("--very-quiet")
-	       :action #'(lambda () (declare (special *maxima-quiet* *display-labels-p*))
-			   (setq *maxima-quiet* t *display-labels-p* nil))
-	       :help-string "Suppress expression labels and Maxima start-up message.")))
+	   (make-cl-option :names '("--very-quiet")
+			   :action #'(lambda () (declare (special *maxima-quiet* *display-labels-p*))
+					     (setq *maxima-quiet* t *display-labels-p* nil))
+			   :help-string "Suppress expression labels and Maxima start-up message.")
+	   (make-cl-option :names '("-X" "--lisp-options")
+			   :argument "<Lisp options>"
+			   :action #'(lambda (&rest opts)
+				       (format t "Lisp options: ~A" opts))
+			   :help-string "Options to be given to the underlying Lisp")
+			   ))
     (process-args (get-application-args) maxima-options))
   (values input-stream batch-flag))
 
@@ -649,6 +695,7 @@ When one changes, the other does too."
   (in-package :maxima)
   (setf *load-verbose* nil)
   (setf *debugger-hook* #'maxima-lisp-debugger)
+  (setf *print-circle* t) ;; "assume" db contains circular objects
   #+ccl (setf ccl::*invoke-debugger-hook-on-interrupt* t)
   (let ((input-stream *standard-input*)
 	(batch-flag nil))
@@ -713,6 +760,16 @@ When one changes, the other does too."
   (declare (ignore dummy))
   $help)
 
+(eval-when
+	#+gcl (load eval)
+	#-gcl (:load-toplevel :execute)
+    (let ((context '$global))
+      (declare (special context))
+      (mapc #'(lambda (x) (kind x '$constant) (setf (get x 'sysconst) t))
+            '($%pi $%i $%e $%phi %i $%gamma ;numeric constants
+                   $inf $minf $und $ind $infinity ;pseudo-constants
+                   t nil)))) ;logical constants (Maxima names: true, false)
+
 ;;; Now that all of maxima has been loaded, define the various lists
 ;;; and hashtables of builtin symbols and values.
 
