@@ -457,10 +457,10 @@
 (defun spc1 (x)
   (cond ((mlogp x) (putonloglist x))
 	((and (mexptp x) (not (eq (cadr x) '$%e)))
-	 ($exp (list '(mtimes)
-		     (caddr x)
-		     (putonloglist (list '(%log simp ratsimp)
-					 (cadr x))))))
+         ($exp-form (list '(mtimes)
+                          (caddr x)
+                          (putonloglist (list '(%log simp ratsimp)
+                                              (cadr x))))))
 	(t x)))
 
 (defun putonloglist (l)
@@ -486,7 +486,8 @@
   (when (and (m$exp? x)
 	     (not (atom (setq y (caddr x))))
 	     (mplusp (setq y (expand1 (if *var ($partfrac y *var) y) 10 10))))
-    (setq y (cons '(mtimes) (mapcar #'(lambda (z) ($ratsimp ($exp z))) (cdr y))))
+    (setq y (cons '(mtimes) 
+                  (mapcar #'(lambda (z) ($ratsimp ($exp-form z))) (cdr y))))
     (radsubst (rform y) (rget v))
     (dsubsta y x varlist)))
 
@@ -513,13 +514,13 @@
     (destructuring-let* ((((c1 . c) . r) (rzcontent (rform v)))
 			 (g (assoc r gcdlist :test #'equal)))
       (cond (g (setf (cadr g) (plcm c (cadr g)))
-	       (push (list ($exp (div* v c1)) c) (cddr g)))
-	    (t (push (list r c (list ($exp (div* v c1)) c)) gcdlist)))))
+               (push (list ($exp-form (div* v c1)) c) (cddr g)))
+            (t (push (list r c (list ($exp-form (div* v c1)) c)) gcdlist)))))
   (dolist (g gcdlist)
     (let ((rd (rdis (car g))))
       (when (and (mlogp rd) (memalike (cadr rd) oldvarlist))
 	(push (list (cadr rd) 1) (cddr g)))
-      (rplaca g ($exp (div rd (cadr g))))))
+      (rplaca g ($exp-form (div rd (cadr g))))))
   (spc5b gcdlist oldvarlist oldgenvar))
 
 ;;(DEFUN SPC5B (V VARLIST GENVAR)
