@@ -149,7 +149,9 @@
   
 ;;; REALP
 
-;; GCL doesn't have the REAL class!
+;; GCL doesn't have the REAL class!  But since a real is a rational or
+;; float, we can fake it by defining methods on rationals and floats
+;; for gcl.
 #-gcl
 (defmethod realp ((x cl:real))
   t)
@@ -433,7 +435,7 @@
 		 :real (maxima::bcons
 			(maxima::fpdifference (cdr (real-value a))
 					      (cdr (intofp (realpart b)))))
-		 :imag (maxima::bcons (maxima::fpminus (intofp (imagpart b))))))
+		 :imag (maxima::bcons (maxima::fpminus (cdr (intofp (imagpart b)))))))
 
 (defmethod two-arg-- ((a cl:float) (b bigfloat))
   (make-instance 'bigfloat
@@ -1102,10 +1104,23 @@
 		   :real (maxima::$realpart result)
 		   :imag (maxima::$imagpart result))))
 
-;; Really want type real, but gcl doesn't like that.
-(defmethod two-arg-atan ((a #-gcl real #+gcl number) (b #-gcl real #+gcl number))
+;; Really want type real, but gcl doesn't like that.  Define methods for rational and float
+#-gcl
+(defmethod two-arg-atan ((a real) (b real))
   (cl:atan a b))
 
+#+gcl
+(progn
+  (defmethod two-arg-atan ((a cl:float) (b cl:float))
+    (cl:atan a b))
+  (defmethod two-arg-atan ((a cl:rational) (b cl:rational))
+    (cl:atan a b))
+  (defmethod two-arg-atan ((a cl:float) (b cl:rational))
+    (cl:atan a b))
+  (defmethod two-arg-atan ((a cl:rational) (b cl:float))
+    (cl:atan a b))
+  )
+
 (defmethod two-arg-atan ((a bigfloat) (b bigfloat))
   (make-instance 'bigfloat
 		 :real (maxima::bcons
@@ -1237,17 +1252,39 @@
 
 ;; We really want a real type, but gcl doesn't like it, so use number
 ;; instead.
-(defmethod one-arg-complex ((a #-gcl real #+gcl number))
+#-gcl
+(defmethod one-arg-complex ((a real))
   (cl:complex a))
 
+#+gcl
+(progn
+(defmethod one-arg-complex ((a cl:float))
+  (cl:complex a))
+(defmethod one-arg-complex ((a cl:rational))
+  (cl:complex a))
+)
+
 (defmethod one-arg-complex ((a bigfloat))
   (make-instance 'complex-bigfloat
 		 :real (real-value a)
 		 :imag (intofp 0)))
 
-(defmethod two-arg-complex ((a #-gcl real #+gcl number) (b #-gcl real #+gcl number))
+#-gcl
+(defmethod two-arg-complex ((a real) (b real))
   (cl:complex a b))
 
+#+gcl
+(progn
+(defmethod two-arg-complex ((a cl:float) (b cl:float))
+  (cl:complex a b))
+(defmethod two-arg-complex ((a cl:rational) (b cl:rational))
+  (cl:complex a b))
+(defmethod two-arg-complex ((a cl:float) (b cl:rational))
+  (cl:complex a b))
+(defmethod two-arg-complex ((a cl:rational) (b cl:float))
+  (cl:complex a b))
+)
+
 (defmethod two-arg-complex ((a bigfloat) (b bigfloat))
   (make-instance 'complex-bigfloat
 		 :real (real-value a)
@@ -1413,6 +1450,24 @@
 	  (* a b)
 	  (exp (* (bigfloat b) (log a))))))
 
+;; Handle a^b a little more carefully because the result is known to
+;; be real when a is real and b is an integer.
+(defmethod expt ((a bigfloat) (b integer))
+  (cond ((zerop b)
+	 (bigfloat 1))
+	((and (zerop a) (plusp b))
+	 ;; 0^b, for positive b
+	 (* a b))
+	((eql b 1) a)
+	((eql b -1) (/ a))
+	((minusp a)
+	 ;; a^b = exp(b*log(|a|) + %i*%pi*b)
+	 ;;     = exp(b*log(|a|))*exp(%i*%pi*b)
+	 ;;     = (-1)^b*exp(b*log(|a|))
+	 (* (exp (* b (log (abs a))))
+	    (if (oddp b) -1 1)))
+	(t
+	 (exp (* b (log a))))))
 
 ;;; TO - External
 ;;;
@@ -1804,7 +1859,7 @@
 	   ;; (coerce foo 'complex-bigfloat).  Foo has to be a real or complex
 	   (cond ((typep obj 'real)
 		  (bigfloat obj 0))
-		 ((typep obj 'complex)
+		 ((typep obj 'cl:complex)
 		  (bigfloat obj))
 		 ((typep obj 'bigfloat)
 		  (bigfloat obj 0))
