@@ -33,6 +33,9 @@
 
 (defmacro pair (al bl) `(mapcar #'cons ,al ,bl))
 
+;; internal representation of risch expressions: list with canonical rational
+;; expression (CRE) as first element, standard maxima expressions as remaining
+;; elements.  risch expression is sum of CRE and remaining elements.
 (defmacro rischzero () ''((0 . 1) 0))
 
 (defun rischnoun (exp1 &optional (exp2 exp1 exp2p))
@@ -44,6 +47,10 @@
        (gl genvar (cdr gl)))
       ((null (cdr vl)) (car gl))))
 
+;; test whether CRE p is constant with respect to variable of integration.
+;; requires variables in varlist and genvar
+;; to be ordered as by intsetup, with var of integration ordered before
+;; any other expressions that contain it.
 (defun risch-pconstp (p)
   (or (pcoefp p) (pointergp mainvar (car p))))
 
@@ -51,6 +58,7 @@
   (setq r (ratfix r))
   (and (risch-pconstp (car r)) (risch-pconstp (cdr r))))
 
+;; adds two risch expressions (defined above).
 (defun rischadd (x y)
   (destructuring-let (((a . b) x) ((c . d) y))
     (cons (r+ a c) (append b d))))
@@ -343,6 +351,18 @@
 				      (getfncoeff (cdr a) form)))
 	((eq (caaar a) 'mtimes)
 	 (destructuring-let (((coef . newfn) (getfnsplit (cdar a))))
+           ;; (car a) is a mtimes expression. We insert coef and newfn as the
+           ;; new arguments to the mtimes expression. This causes problems if
+           ;;   (1) coef is a mtimes expression too and
+           ;;   (2) (car a) has already a simp flag
+           ;; We get a nested mtimes expression, which does not simplify.
+	   ;; We comment out the following code (DK 09/2009):
+           ;; (setf (cdar a) (list coef newfn))
+	   
+	   ;; Insert a complete mtimes expression without simpflag.
+           ;; Nested mtimes expressions simplify further.
+           (setf (car a) (list '(mtimes) coef newfn))
+	   
 	   (setf (cdar a) (list coef newfn))
 	   (cond ((zerop1 coef) (getfncoeff (cdr a) form))
 		 ((and (matanp newfn) (member '$%i varlist :test #'eq))
@@ -436,7 +456,7 @@
 		       (narg (remabs (cadr nlog)))
 		       (varlist varlist)
 		       (genvar genvar)
-		       (rn (rform narg))
+		       (rn (rform narg))	;; can add new vars to varlist
 		       (ro (rform (cadr olog)))
 		       (var (caar ro))
 		       ((j . k) (ratreduce (pdegree (car rn) var) (pdegree (car ro) var)))
@@ -451,8 +471,8 @@
 		 coef (div coef (f* j degree))
 		 olog (mul j olog))))
     (desetq (rc . rd) (ratdivide rn ro))
-    (cond ((and (risch-constp rc)
-		(risch-constp rd))
+    (cond ((and (freeof intvar (rdis rc))	;; can't use risch-constp because varlist
+		(freeof intvar (rdis rd)))	;; is not set up with vars in correct order.
 	   (setq narg ($ratsimp (sub 1 (div narg (rdis rd)))))
 	   (mul* coef (power -1 (1+ degree))
 		 `((mfactorial) ,degree)
@@ -470,8 +490,9 @@
     (setq y (get var 'leadop))
     (cond ((and (not (pzerop (ratnumerator f)))
 		(risch-constp (setq l (ratqu a f))))
-	   (cond (flag
-		  (list (r* l (cons (list expg n 1) 1)) 0))
+	   (cond (flag		;; multiply in expg^n - n may be negative
+		  (list (r* l (ratexpt (cons (list expg 1 1) 1) n))
+			0))
 		 (t l)))
 	  ((eq y intvar)
 	   (rischexpvar nil flag (list f a expg n)))
