@@ -14,7 +14,7 @@
 
 (load-macsyma-macros ratmac)
 
-(declare-top (special rischpf genvar $savefactors checkfactors
+(declare-top (special genvar $savefactors checkfactors
 		      exp var $factorflag $ratfac $logabs $expop $expon
 		      $keepfloat ratform rootfactor pardenom $algebraic
 		      wholepart parnumer varlist logptdx switch1))
@@ -138,7 +138,7 @@
 
 (defun integrate-use-rootsof (f q variable &aux qprime ff qq (dummy (make-param)) lead)
   ;; p2e is squarefree in polynomial in cre form p1e is lower degree
-  (setq lead (p-lc q))
+  (setq lead (disrep (p-lc q)))
   (setq qprime (disrep (pderivative q (p-var q))))
   (setq ff (disrep f) qq (disrep q))
   `((%lsum) ((mtimes)
@@ -199,11 +199,12 @@
 		      (ratdifference (cons (pexpt (polcoef p2e 1) 2) 1)
 				     (ratti 4 (ratti (polcoef p2e 2) (polcoef p2e 0) t) t)))
      (setq a2e (ratti (polcoef p2e (pdegree p2e var)) 2 t))
-     (if (not (free (setq xx (simplify (disrep discrim))) '$%i)) (go pos))
-     (setq sign (npask xx))
-     (cond ((eq sign '$negative) (go e30))
-	   ((eq sign '$zero) (go zip)))
-     pos	(setq a1e (ratsqrt discrim))
+     (setq xx (simplify (disrep discrim)))
+     (when (equal ($imagpart xx) 0)
+       (setq sign (npask xx))
+       (cond ((eq sign '$negative) (go e30))
+	     ((eq sign '$zero) (go zip))))
+     (setq a1e (ratsqrt discrim))
      (setq a3e (logmabs
 		(list '(mquotient)
 		      (list '(mplus)
