@@ -26,7 +26,7 @@
 
 ;; subtitle polylogarithm routines
 
-(declare-top (special $zerobern ivars key-vars tlist))
+(declare-top (special $zerobern ivars key-vars tlist %e-val))
 
 (defun lisimp (exp vestigial z)
   (declare (ignore vestigial))
@@ -378,7 +378,7 @@
 
 (defun plygam-ord (subl)
   (if (equal (car subl) -1) (ncons (rcone))
-      `((,(- (1+ (car subl))) . 1))))
+      `((,(m- (m1+ (car subl))) . 1))))
 
 (defun plygam-pole (a c func)
   (if (rcmintegerp c)
