@@ -23,28 +23,17 @@
 
 (defmvar mopl nil)
 
-(declare-top  (special $debugmode $lasttime $disptime
-		       bindlist loclist errset $labels linelable $filesize
-		       st rephrase $dispflag refchkl baktrcl
-		       dskfnp dsksavep *rset ^q lf tab ff cr
-		       $values $functions $arrays $aliases $gradefs $dependencies
-		       $rules $props $ratvars $ratvarswitch *mdebug* errbrksw errcatch
-		       varlist genvar $device $filename $filenum
-		       $gensumnum checkfactors $features featurel $backtrace
-		       $weightlevels tellratlist $dontfactor $infolists
-		       $dskall allbutl lisperrprint dcount thistime
-		       $nolabels dispflag saveno mcatch brklvl savefile
-		       $%% $error *in-translate-file*
-		       lessorder greatorder $errorfun mbreak reprint pos $strdisp
-		       $dskuse smart-tty more-^w oldst
-		       $loadprint opers *ratweights $ratweights quitmsg
-		       loadf $grind $stringdisp $lispdisp defaultf
-		       state-pdl command
-		       transp $contexts $setcheck $macros
-		       autoload))
-
-(defvar $prompt '_)
-
+(declare-top  (special
+		       bindlist loclist errset *rset ^q lf tab ff cr
+		       $values $functions $arrays $gradefs $dependencies
+		       $rules $props $ratvars $ratvarswitch
+		       varlist genvar $filename
+		       $gensumnum checkfactors $features featurel
+		       $weightlevels tellratlist $dontfactor
+		       dispflag savefile $%% $error smart-tty
+		       opers *ratweights $ratweights
+		       $stringdisp $lispdisp defaultf command
+		       transp $contexts $setcheck $macros autoload))
 
 (mapc #'(lambda (x) (putprop (car x) (cadr x) 'opalias))
       '((+ $+) (- $-) (* $*) (// $//) (^ $^) (|.| |$.|) (< $<) (= $=)
@@ -57,51 +46,60 @@
 			       (t (cadr x)))))
       '((tab #\tab) (lf #\linefeed) (ff #\page) (cr #\return) (sp #\space)))
 
-(setq $lasttime '((mlist) 0 0)
-      thistime 0
-      $disptime nil)
+(defvar thistime 0)
+(defvar refchkl nil)
+(defvar *mdebug* nil)
+(defvar baktrcl nil)
+(defvar errbrksw nil)
+(defvar mbreak nil)
+(defvar errcatch nil)
+(defvar mcatch nil)
+(defvar brklvl -1)
+(defvar allbutl nil)
+(defvar loadf nil)
+(defvar lessorder nil)
+(defvar greatorder nil)
+(defvar *in-translate-file* nil)
+(defvar linelable nil)
+(defvar rephrase nil)
+(defvar st nil)
+(defvar oldst nil)
+(defvar reprint nil)
+(defvar pos nil)
+(defvar dcount 0)
+(defvar dskfnp nil)
+(defvar dsksavep nil)
+(defvar saveno 0)
+(defvar quitmsg  " ")
+(defvar more-^w nil)
+(defvar lisperrprint t)
 
-(setq $strdisp t $grind nil)
+(defvar state-pdl (ncons 'lisp-toplevel))
 
-(setq refchkl nil
-      *mdebug* nil
-      baktrcl nil
-      errbrksw nil
-      mbreak nil
-      $errorfun nil
-      errcatch nil
-      mcatch nil
-      brklvl -1
-      allbutl nil
-      loadf nil
-      $backtrace '$backtrace)
+(defmvar $filenum 0)
+(defmvar $storenum 1000.)
+(defmvar $dskall t)
+(defmvar $errorfun nil)
+(defmvar $disptime nil)
+(defmvar $strdisp t)
+(defmvar $grind nil)
+(defmvar $prompt '_)
+(defmvar $backtrace '$backtrace)
+(defmvar $debugmode nil)
+(defmvar $pagepause nil)
+(defmvar $poislim 5)
+(defmvar $loadprint nil)
+(defmvar $nolabels nil)
+(defmvar $aliases '((mlist simp)))
 
-(setq *in-translate-file* nil)
+(defmvar $infolists
+  '((mlist simp) $labels $values $functions $macros $arrays
+                 $myoptions $props $aliases $rules $gradefs
+                 $dependencies $let_rule_packages $structures))
 
-(setq $debugmode nil $pagepause nil $poislim 5)
-
-(setq $loadprint nil)
-
-(setq $nolabels nil $aliases '((mlist simp)) lessorder nil greatorder nil)
-
-(setq $infolists '((mlist simp) $labels $values $functions $macros $arrays
-		   $myoptions $props $aliases $rules $gradefs
-		   $dependencies $let_rule_packages $structures))
-
-(setq $labels (list '(mlist simp)))
-
-(setq $dskuse nil $device '$dsk $dispflag t linelable nil)
-
-(setq rephrase nil st nil oldst nil reprint nil pos nil)
-
-(setq dcount 0 $filenum 0 $storenum 1000. $filesize 16. $dskall t
-      dskfnp nil dsksavep nil saveno 0)
-
-(setq quitmsg  " "
-      more-^w nil
-      lisperrprint t)
-
-(setq state-pdl (ncons 'lisp-toplevel))
+(defmvar $labels (list '(mlist simp)))
+(defmvar $device '$dsk)
+(defmvar $dispflag t)
 
 (defmvar $% '$% "The last out-line computed, corresponds to lisp *"
 	 no-reset)
@@ -143,16 +141,27 @@
 
 (defmvar $showtime nil)
 
-(defmfun meval* (test)
-  (let (refchkl baktrcl checkfactors)
-    (prog2 (if $ratvarswitch (setq varlist (cdr $ratvars)))
-	(meval test)
-      (clearsign))))
+;(defmfun meval* (test)
+;  (let (refchkl baktrcl checkfactors)
+;    (prog2 (if $ratvarswitch (setq varlist (cdr $ratvars)))
+;	(meval test)
+;      (clearsign))))
+
+;; This version of meval* makes sure that the facts from the global variable
+;; locals are cleared with a call to clearsign. The facts are added by asksign
+;; and friends. meval* function is only used for top level evaluations.
+;; For other cases the function meval can be used.
+
+(defun meval* (expr)
+  ;; Make sure that clearsign is called after the evaluation.
+  (unwind-protect
+    (let (refchkl baktrcl checkfactors)
+      (if $ratvarswitch (setq varlist (cdr $ratvars)))
+      (meval expr))
+    ;; Clear the facts from asksign and friends.
+    (clearsign)))
 
 (defmfun makelabel (x)
-  (when (and $dskuse (not $nolabels) (> (incf dcount) $filesize))
-    (setq dcount 0)
-    (dsksave))
   (setq linelable ($concat '|| x $linenum))
   (unless $nolabels
     (when (or (null (cdr $labels))
@@ -213,19 +222,23 @@
 ; Following GENERIC-AUTOLOAD is copied from orthopoly/orthopoly-init.lisp.
 ; Previous version didn't take Clisp, CMUCL, or SBCL into account.
 
+(defvar *autoloaded-files* ())
+
 (defun generic-autoload (file &aux type)
-  (setq file (pathname (cdr file)))
-  (setq type (pathname-type file))
-  (let ((bin-ext #+gcl "o"
-	 #+cmu (c::backend-fasl-file-type c::*target-backend*)
-	 #+clisp "fas"
-	 #+allegro "fasl"
-	 #+openmcl (pathname-type ccl::*.fasl-pathname*)
-	 #+lispworks (pathname-type (compile-file-pathname "foo.lisp"))
-	 #-(or gcl cmu clisp allegro openmcl lispworks) ""))
-    (if (member type (list bin-ext "lisp" "lsp")  :test 'equalp)
-      #-sbcl (load file) #+sbcl (with-compilation-unit nil (load file))
-      ($load file))))
+  (unless (member file *autoloaded-files* :test #'equal)
+    (push file *autoloaded-files*)
+    (setq file (pathname (cdr file)))
+    (setq type (pathname-type file))
+    (let ((bin-ext #+gcl "o"
+		   #+cmu (c::backend-fasl-file-type c::*target-backend*)
+		   #+clisp "fas"
+		   #+allegro "fasl"
+		   #+openmcl (pathname-type ccl::*.fasl-pathname*)
+		   #+lispworks (pathname-type (compile-file-pathname "foo.lisp"))
+		   #-(or gcl cmu clisp allegro openmcl lispworks) ""))
+      (if (member type (list bin-ext "lisp" "lsp")  :test 'equalp)
+	  #-sbcl (load file) #+sbcl (with-compilation-unit nil (load file))
+	  ($load file)))))
 
 (defvar autoload 'generic-autoload)
 
@@ -297,6 +310,7 @@
   `(if specp (setq filepos (filepos file-obj))))
 
 (defmspec $kill (form)
+  (clear)	;; get assume db into consistent state
   (mapc #'kill1 (cdr form))
   '$done)
 
@@ -345,7 +359,8 @@
 	    (remprop x 'operators)))
       (when (member x (cdr $props) :test #'equal)
 	(remprop x 'sp2)
-	(killframe x))
+	(killframe x)
+	(i-$remove (list x $features)))
       (let ((y (get x 'op)))
 	(when (and y (not (member y mopl :test #'equal)) (member y (cdr $props) :test #'equal))
 	  (kill-operator x)))
@@ -714,8 +729,10 @@
 (defmfun alias (x y)
   (cond ((nonsymchk x '$alias))
 	((nonsymchk y '$alias))
-	((not (eq (getcharn x 1) #\$))
-	 (merror "-ed symbols may not be aliased. ~M" x))
+        ((eq x y) y) ; x is already the alias of y
+; Not needed. We return the alias immediately if we already have one.
+;	((not (eq (getcharn x 1) #\$))
+;	 (merror "-ed symbols may not be aliased. ~M" x))
 	((get x 'reversealias)
 	 (if (not (eq x y))
 	     (merror "~M already is aliased." x)))
@@ -838,7 +855,9 @@
 
 (mapc #'(lambda (x) (putprop (car x) (cadr x) 'alias))
       '(($ratcoeff $ratcoef) ($ratnum $ratnumer) ($true t)
-	($binom %binomial) ($derivative $diff) ($prod $product)
+; Cut out the alias binom for binomial
+;	($binom %binomial)
+        ($derivative $diff) ($prod $product)
 	($bothcoeff $bothcoef)))
 
 (defmfun amperchk (name)
@@ -962,13 +981,11 @@
 
 (defmspec $time (l)
   (setq l (cdr l))
-  ;;(format t "~&Time:")
   (cons '(mlist simp)
 	(mapcar
 	 #'(lambda (x)
 	     (or (and (symbolp x)
-		      (setq x (or (get x 'time)
-				  (and (eq x '$%) (cons (cadr $lasttime) (caddr $lasttime)))))
+		      (setq x (get x 'time))
 		      (if (= (cdr x) 0)
 			  (car x)
 			  (list '(mlist simp) (car x) (cdr x))))
@@ -1022,28 +1039,30 @@
 (defmspec $status (form)
   (setq form (cdr form))
   (let* ((keyword (car form))
-	 (feature (cadr form)))
-    (assert (symbolp keyword))
-    (assert (or (stringp feature) (symbolp feature)))
+         (feature (cadr form)))
+    (when (not (symbolp keyword))
+      (merror (intl:gettext "status: First argument must be a symbol.")))
+    (when (not (or (stringp feature) (symbolp feature)))
+      (merror
+        (intl:gettext "status: Second argument must be symbol or a string.")))
     (case keyword
       ($feature (cond ((null feature) (dollarify *features*))
-		      ((member (intern
-                         (if (stringp feature)
-                           (maybe-invert-string-case feature)
-                           (symbol-name (fullstrip1 feature)))
-                         'keyword)
-			     *features* :test #'equal) t)))
-      ($status '((mlist simp) $feature $status))
-      (t (merror "Unknown argument - `status':~%~M" keyword)))))
+                      ((member (intern (if (stringp feature)
+                                           (maybe-invert-string-case feature)
+                                           (symbol-name (fullstrip1 feature)))
+                                       'keyword)
+                               *features* :test #'equal) t)))
+      (t (merror (intl:gettext "status: Unknown argument: ~M~%") keyword)))))
 
-(defquote $sstatus (status-function item)
-  (cond ((equal status-function '$feature)
-	 (pushnew ($mkey item) *features*) t)
-	((equal status-function '$nofeature)
-	 (setq *features* (delete ($mkey item) *features*)) t)
-	(t (error "know only how to set and remove feature status"))))
+(defquote $sstatus (keyword item)
+  (cond ((equal keyword '$feature)
+         (pushnew ($mkey item) *features*) t)
+        ((equal keyword '$nofeature)
+         (setq *features* (delete ($mkey item) *features*)) t)
+        (t
+         (merror (intl:gettext "sstatus: Unknown argument: ~M~%") keyword))))
 
-(dolist (l '($sqrt $sin $cos $tan $log $plog $sec $csc $cot $sinh $cosh
+(dolist (l '($sin $cos $tan $log $plog $sec $csc $cot $sinh $cosh
 	     $tanh $sech $csch $coth $asin $acos $atan $acot $acsc $asec $asinh
 	     $acosh $atanh $acsch $asech $acoth $binomial $gamma $genfact $del))
   (let ((x ($nounify l)))
