@@ -87,6 +87,14 @@
 (defprop %acsch simp-%acsch operators)
 (defprop %asech simp-%asech operators)
 
+;;; The trigonometric functions distribute of lists, matrices and equations.
+
+(dolist (x '(%sin   %cos   %tan   %cot   %csc   %sec
+             %sinh  %cosh  %tanh  %coth  %csch  %sech
+             %asin  %acos  %atan  %acot  %acsc  %asec
+             %asinh %acosh %atanh %acoth %acsch %asech))
+  (setf (get x 'distribute_over) '(mlist $matrix mequal)))
+
 (defun domain-error (x f)
   (merror "The number ~:M isn't in the domain of ~A" (complexify x) f))
 
@@ -264,7 +272,7 @@
 		     (if y y (domain-error x 'acoth)))))
 
   (frob %mabs #'cl:abs)
-  (frob $exp #'cl:exp)
+  (frob %exp #'cl:exp)
   (frob mexpt #'cl:expt)
   (frob %sqrt #'cl:sqrt)
   (frob %log #'(lambda (x)
@@ -282,7 +290,7 @@
   (frob $imagpart #'cl:imagpart)
   (frob $max #'cl:max)
   (frob $min #'cl:min)
-  (frob %signnum #'cl:signum)
+  (frob %signum #'cl:signum)
   (frob $atan2 #'cl:atan)
   (frob %log #'(lambda (x)
 		 (let ((y (ignore-errors (cl:log x))))
